<?php
/**
*
* @package ucp
* @version $Id$
* @copyright (c) 2005 phpBB Group
* @license http://opensource.org/licenses/gpl-license.php GNU Public License
*
*/

/**
* @ignore
*/
if (!defined('IN_PHPBB'))
{
	exit;
}

/**
* ucp_register
* Board registration
* @package ucp
*/
class ucp_register
{
	var $u_action;

	function main($id, $mode)
	{
		global $config, $db, $user, $auth, $template, $phpbb_root_path, $phpEx;

		//
		if ($config['require_activation'] == USER_ACTIVATION_DISABLE)
		{
			trigger_error('UCP_REGISTER_DISABLE');
		}

		include($phpbb_root_path . 'includes/functions_profile_fields.' . $phpEx);

		$coppa			= (isset($_REQUEST['coppa'])) ? ((!empty($_REQUEST['coppa'])) ? 1 : 0) : false;
		$agreed			= (!empty($_POST['agreed'])) ? 1 : 0;
		$submit			= (isset($_POST['submit'])) ? true : false;
		$change_lang	= request_var('change_lang', '');
		$user_lang		= request_var('lang', $user->lang_name);

		if ($agreed)
		{
			add_form_key('ucp_register');
		}
		else
		{
			add_form_key('ucp_register_terms');
		}

		if ($change_lang || $user_lang != $config['default_lang'])
		{
			$use_lang = ($change_lang) ? basename($change_lang) : basename($user_lang);

			if (!validate_language_iso_name($use_lang))
			{
				if ($change_lang)
				{
					$submit = false;

					// Setting back agreed to let the user view the agreement in his/her language
					$agreed = (empty($_GET['change_lang'])) ? 0 : $agreed;
				}

				$user->lang_name = $user_lang = $use_lang;
				$user->lang = array();
				$user->data['user_lang'] = $user->lang_name;
				$user->add_lang(array('common', 'ucp'));
			}
			else
			{
				$change_lang = '';
				$user_lang = $user->lang_name;
			}
		}


		$cp = new custom_profile();

		$error = $cp_data = $cp_error = array();

		if (!$agreed || ($coppa === false && $config['coppa_enable']) || ($coppa && !$config['coppa_enable']))
		{
			$add_lang = ($change_lang) ? '&amp;change_lang=' . urlencode($change_lang) : '';
			$add_coppa = ($coppa !== false) ? '&amp;coppa=' . $coppa : '';

			$s_hidden_fields = array(
				'change_lang'	=> $change_lang,
			);

			// If we change the language, we want to pass on some more possible parameter.
			if ($change_lang)
			{
				// We do not include the password
				$s_hidden_fields = array_merge($s_hidden_fields, array(
					'username'			=> utf8_normalize_nfc(request_var('username', '', true)),
					'email'				=> strtolower(request_var('email', '')),
					'email_confirm'		=> strtolower(request_var('email_confirm', '')),
					'lang'				=> $user->lang_name,
					'tz'				=> request_var('tz', (float) $config['board_timezone']),
				));

			}

			// Checking amount of available languages
			$sql = 'SELECT lang_id
				FROM ' . LANG_TABLE;
			$result = $db->sql_query($sql);

			$lang_row = array();
			while ($row = $db->sql_fetchrow($result))
			{
				$lang_row[] = $row;
			}
			$db->sql_freeresult($result);

			if ($coppa === false && $config['coppa_enable'])
			{
				$now = getdate();
				$coppa_birthday = $user->format_date(mktime($now['hours'] + $user->data['user_dst'], $now['minutes'], $now['seconds'], $now['mon'], $now['mday'] - 1, $now['year'] - 13), $user->lang['DATE_FORMAT']);
				unset($now);

				$template->assign_vars(array(
					'S_LANG_OPTIONS'	=> (sizeof($lang_row) > 1) ? language_select($user_lang) : '',
					'L_COPPA_NO'		=> sprintf($user->lang['UCP_COPPA_BEFORE'], $coppa_birthday),
					'L_COPPA_YES'		=> sprintf($user->lang['UCP_COPPA_ON_AFTER'], $coppa_birthday),

					'U_COPPA_NO'		=> append_sid("{$phpbb_root_path}ucp.$phpEx", 'mode=register&amp;coppa=0' . $add_lang),
					'U_COPPA_YES'		=> append_sid("{$phpbb_root_path}ucp.$phpEx", 'mode=register&amp;coppa=1' . $add_lang),

					'S_SHOW_COPPA'		=> true,
					'S_HIDDEN_FIELDS'	=> build_hidden_fields($s_hidden_fields),
					'S_UCP_ACTION'		=> append_sid("{$phpbb_root_path}ucp.$phpEx", 'mode=register' . $add_lang),
				));
			}
			else
			{
				$template->assign_vars(array(
					'S_LANG_OPTIONS'	=> (sizeof($lang_row) > 1) ? language_select($user_lang) : '',
					'L_TERMS_OF_USE'	=> sprintf($user->lang['TERMS_OF_USE_CONTENT'], $config['sitename'], generate_board_url()),

					'S_SHOW_COPPA'		=> false,
					'S_REGISTRATION'	=> true,
					'S_HIDDEN_FIELDS'	=> build_hidden_fields($s_hidden_fields),
					'S_UCP_ACTION'		=> append_sid("{$phpbb_root_path}ucp.$phpEx", 'mode=register' . $add_lang . $add_coppa),
					)
				);
			}
			unset($lang_row);

			$this->tpl_name = 'ucp_agreement';
			return;
		}


		// The CAPTCHA kicks in here. We can't help that the information gets lost on language change. 
		if ($config['enable_confirm'])
		{
			include($phpbb_root_path . 'includes/captcha/captcha_factory.' . $phpEx);
			$captcha =& phpbb_captcha_factory::get_instance($config['captcha_plugin']);
			$captcha->init(CONFIRM_REG);
		}

		$is_dst = $config['board_dst'];
		$timezone = $config['board_timezone'];

		$data = array(
			'username'			=> utf8_normalize_nfc(request_var('username', '', true)),
			'new_password'		=> request_var('new_password', '', true),
			'password_confirm'	=> request_var('password_confirm', '', true),
			'email'				=> strtolower(request_var('email', '')),
			'email_confirm'		=> strtolower(request_var('email_confirm', '')),
			'lang'				=> basename(request_var('lang', $user->lang_name)),
			'tz'				=> request_var('tz', (float) $timezone),
		);



		// Begin: cleantalk.ru moderation
		$user->add_lang('mods/info_acp_cleantalk');

		$template->assign_vars(array(
			'CT_ENTER_CODE' => $user->lang['CT_ENTER_CODE'],
			'CT_ENTER_CODE_EXPLAIN' => $user->lang['CT_ENTER_CODE_EXPLAIN'],
			'CT_GO_TO_RIGESTER_FORM' => $user->lang['CT_GO_TO_RIGESTER_FORM']
		));

		$ct_status = 0;

		if ($config['ct_newuser'] && $config['ct_enable'] && $config['ct_sms_check'])
		{
			if (isset($user->data['ct_register_form_data']) && !empty($user->data['ct_register_form_data']))
			{
				if (($user->data['ct_register_form_data_time'] + 600) > time())
				{
					if (request_var('pf_ct_phone', '', true))
					{
						$ct_data = unserialize($user->data['ct_register_form_data']);
						$data = array_merge($ct_data['data'], $data);
					}
					else
					{
						$ct_data = unserialize($user->data['ct_register_form_data']);
						$data = $ct_data['data'];
					}
				}
				else
				{
					$user->data['ct_register_form_data'] = null;
				}
			}

			if ($submit)
			{
				if (request_var('ct_sms_code', '', true))
				{
					$ct_status			 = 2;
					$data['ct_sms_code'] = request_var('ct_sms_code', '', true);
				}
				if ($ct_status === 0)
				{
					$ct_status			 = 1;
					$data['ct_phone']	 = request_var('pf_ct_phone', '', true);
				}
			}

			$tpl_snippet = $cp->process_field_row('change', array(
				'field_value' => '',
				'lang_default_value' => '',
				'field_type' => 2,
				'field_ident' => 'ct_phone'
					));

			$template->assign_block_vars('profile_fields', array(
				'LANG_NAME' => $user->lang['CT_PHONE_NUMBER'],
				'LANG_EXPLAIN' => $user->lang['CT_PHONE_NUMBER_TEXT'],
				'FIELD' => $tpl_snippet,
				'FIELD_ID' => 'ct_',
				'S_REQUIRED' => 1
			));
		}
		if ($ct_status == 0)
		{
			$data['ct_phone'] = null;
		}

		// End: cleantalk.ru moderation
		// Check and initialize some variables if needed
		if ($submit)
		{
			$error = validate_data($data, array(
				'username'			=> array(
					array('string', false, $config['min_name_chars'], $config['max_name_chars']),
					array('username', '')),
				'new_password'		=> array(
					array('string', false, $config['min_pass_chars'], $config['max_pass_chars']),
					array('password')),
				'password_confirm'	=> array('string', false, $config['min_pass_chars'], $config['max_pass_chars']),
				'email'				=> array(
					array('string', false, 6, 60),
					array('email')),
				'email_confirm'		=> array('string', false, 6, 60),
				'tz'				=> array('num', false, -14, 14),
				'lang'				=> array('language_iso_name'),
			));

			if (!check_form_key('ucp_register'))
			{
				$error[] = $user->lang['FORM_INVALID'];
			}

			// Replace "error" strings with their real, localised form
			$error = preg_replace('#^([A-Z_]+)$#e', "(!empty(\$user->lang['\\1'])) ? \$user->lang['\\1'] : '\\1'", $error);

			if ($config['enable_confirm'])
			{
				$vc_response = $captcha->validate($data);
				if ($vc_response !== false)
				{
					$error[] = $vc_response;
				}

				if ($config['max_reg_attempts'] && $captcha->get_attempt_count() > $config['max_reg_attempts'])
				{
					$error[] = $user->lang['TOO_MANY_REGISTERS'];
				}
			}

			// DNSBL check
			if ($config['check_dnsbl'])
			{
				if (($dnsbl = $user->check_dnsbl('register')) !== false)
				{
					$error[] = sprintf($user->lang['IP_BLACKLISTED'], $user->ip, $dnsbl[1]);
				}
			}

			// Begin: cleantalk.ru moderation
			$ct_error_tmp = $error;
			// End: cleantalk.ru moderation
			
			// validate custom profile fields
			$cp->submit_cp_field('register', $user->get_iso_lang_id(), $cp_data, $error);
			
			// Begin: cleantalk.ru moderation
			/*
				Fill $cp_data from phpbb_sesssions.ct_register_form_data_time cause we checked cp fields at step $ct_status = 0|1 
			*/
			if ($ct_status == 2 && !empty($ct_data['cp_data']) && $error != $ct_error_tmp){
				$cp_data = $ct_data['cp_data'];
				$error = $ct_error_tmp;
			}
			// End: cleantalk.ru moderation

			if (!sizeof($error))
			{
				if ($data['new_password'] != $data['password_confirm'])
				{
					$error[] = $user->lang['NEW_PASSWORD_ERROR'];
				}

				if ($data['email'] != $data['email_confirm'])
				{
					$error[] = $user->lang['NEW_EMAIL_ERROR'];
				}
			}
			// Begin: cleantalk.ru moderation
			if (!sizeof($error) && $config['ct_newuser'] && $config['ct_enable'])
			{
				include($phpbb_root_path . 'includes/cleantalk.' . $phpEx);
				include($phpbb_root_path . 'includes/cleantalk.class.' . $phpEx);
				
				$ct_phone = null;
				if ($config['ct_sms_check'])
					$ct_phone = 'not_filled';

				if (isset($data['ct_phone']) && !empty($data['ct_phone']))
					$ct_phone = $data['ct_phone'];
						
				if ($ct_status == 0 || $ct_status == 1)
				{
					
					if (function_exists('get_ct_checkjs'))
					{
						$ct_checkjs = get_ct_checkjs();
					}
					else
					{
						$ct_checkjs = null;
					}

					$submit_time = null;
					if (isset($user->data['ct_submit_time']) && preg_match("/^\d+$/", $user->data['ct_submit_time']))
					{
						$submit_time = time() - $user->data['ct_submit_time'];
					}
					
					if (function_exists('get_sender_info'))
					{
						$sender_info = get_sender_info();
					}
					else
					{
						$sender_info = null;
					}

					$ct_request					 = new CleantalkRequest();
					$ct_request->auth_key		 = $config['ct_auth_key'];
					$ct_request->sender_email	 = $data['email'];
					$ct_request->sender_nickname = $data['username'];
					$ct_request->sender_ip		 = ct_session_ip($user->data['session_ip']);
					$ct_request->agent			 = 'ct-phpbb-' . preg_replace("/(\d)\.(\d)\.(\d)/", "$1$2$3", $config['ct_version']);
					$ct_request->response_lang	 = $config['ct_response_lang'];
					$ct_request->submit_time	 = $submit_time;
					$ct_request->js_on			 = $ct_checkjs;
					$ct_request->tz				 = $data['tz'];
					$ct_request->phone			 = $ct_phone;
					$ct_request->sender_info	= $sender_info;

					$ct					 = new Cleantalk();
					$ct->work_url		 = $config['ct_work_url'];
					$ct->server_url		 = $config['ct_server_url'];
					$ct->server_ttl		 = $config['ct_server_ttl'];
					$ct->server_changed	 = $config['ct_server_changed'];

					$ct_result = $ct->isAllowUser($ct_request);

					if ($ct->server_change)
					{
						set_config('ct_work_url', $ct->work_url);
						set_config('ct_server_ttl', $ct->server_ttl);
						set_config('ct_server_changed', time());
					}

					if ($ct_result->errno > 0 || $ct_result->inactive === 1)
					{
                        // Send account to admin activation
                        $ct_activation_admin = true;
                        $ct_activation_comment = $ct_result->comment;

                        $config['email_enable'] = 1;
                        $config['require_activation'] = USER_ACTIVATION_ADMIN;
                        
                        if ($ct_result->errno > 0)
						    ct_error_mail($ct_result->errstr);
                        
					}
                    if ($ct_result->allow === 0 && !isset($ct_result->sms_allow) && $ct_result->inactive == 0)
					{
						$error[] = $ct_result->comment;
						add_log('user', null, sprintf($user->lang['CT_USER_DENY'], $data['username'], $data['email']) . ' ' . $ct_result->comment);
					}
				}

				if (!sizeof($error) && $config['ct_sms_check'] &&
						($ct_status == 1 || $ct_status == 2))
				{
					// begin: for hidden fields
					$s_hidden_fields = array(
						'agreed' => 'true',
						'change_lang' => 0,
					);

					if ($config['coppa_enable'])
					{
						$s_hidden_fields['coppa'] = $coppa;
					}

					if ($config['enable_confirm'])
					{
						$s_hidden_fields = array_merge($s_hidden_fields, $captcha->get_hidden_fields());
					}
					$s_hidden_fields = build_hidden_fields($s_hidden_fields);
					$confirm_image	 = '';

					// Visual Confirmation - Show images
					if ($config['enable_confirm'])
					{
						$template->assign_vars(array(
							'CAPTCHA_TEMPLATE' => $captcha->get_template(),
						));
					}

					$l_reg_cond = '';
					switch ($config['require_activation'])
					{
						case USER_ACTIVATION_SELF:
							$l_reg_cond = $user->lang['UCP_EMAIL_ACTIVATE'];
							break;

						case USER_ACTIVATION_ADMIN:
							$l_reg_cond = $user->lang['UCP_ADMIN_ACTIVATE'];
							break;
					}

					$template->assign_vars(array(
						'ERROR' => (sizeof($error)) ? implode('<br />', $error) : '',
						'USERNAME' => $data['username'],
						'PASSWORD' => $data['new_password'],
						'PASSWORD_CONFIRM' => $data['password_confirm'],
						'EMAIL' => $data['email'],
						'EMAIL_CONFIRM' => $data['email_confirm'],
						'L_REG_COND' => $l_reg_cond,
						'L_USERNAME_EXPLAIN' => sprintf($user->lang[$config['allow_name_chars'] . '_EXPLAIN'], $config['min_name_chars'], $config['max_name_chars']),
						'L_PASSWORD_EXPLAIN' => sprintf($user->lang[$config['pass_complex'] . '_EXPLAIN'], $config['min_pass_chars'], $config['max_pass_chars']),
						'S_LANG_OPTIONS' => language_select($data['lang']),
						'S_TZ_OPTIONS' => tz_select($data['tz']),
						'S_CONFIRM_REFRESH' => ($config['enable_confirm'] && $config['confirm_refresh']) ? true : false,
						'S_REGISTRATION' => true,
						'S_COPPA' => $coppa,
						'S_HIDDEN_FIELDS' => $s_hidden_fields,
						'S_UCP_ACTION' => append_sid("{$phpbb_root_path}ucp.$phpEx", 'mode=register'),
					));
					// end: for hidden fields

					if (isset($ct_result->sms_allow))
					{
						if ($ct_result->sms_allow == '-1')
						{
							$ct_activation_admin = true;
							$ct_activation_comment = $ct_result->sms_error_text;
							
							$config['email_enable'] = 1;
							$config['require_activation'] = USER_ACTIVATION_ADMIN;
						}
						elseif (isset($ct_result->sms_error_text) && !empty($ct_result->sms_error_text))
						{
							$error[] = $ct_result->sms_error_text;
							$template->assign_vars(array(
								'ERROR' => (sizeof($error)) ? implode('<br />', $error) : ''
							));
							$this->tpl_name	 = 'ucp_register';
						}
						else
						{
							$data['ct_request_id'] = $ct_result->id;

							$data = array_merge($data, array(
								'sender_ip' => $ct_request->sender_ip,
								'submit_time' => $ct_request->submit_time,
								'js_on' => $ct_request->js_on,
									));
							$ct_data['data'] = $data;
							$ct_data['cp_data'] = $cp_data;

							$sql = "UPDATE " . SESSIONS_TABLE . "
								SET ct_register_form_data = '" . $db->sql_escape(serialize($ct_data)) . "',
									ct_register_form_data_time = '" . time() . "'
								WHERE session_id = '" . $db->sql_escape($user->session_id) . "'";
							$db->sql_query($sql);

							$this->tpl_name = 'ucp_check_sms';
							return;
						}
					}
					if ($ct_status == 2)
					{
						if (strlen($data['ct_sms_code']) != 4)
						{
							$error[] = $user->lang['CT_SMS_CODE_ERROR'];
							$template->assign_vars(array(
								'ERROR' => (sizeof($error)) ? implode('<br />', $error) : ''
							));
							$this->tpl_name	 = 'ucp_check_sms';
							return;
						}
						$ct					 = new Cleantalk();
						$ct->work_url		 = $config['ct_work_url'];
						$ct->server_url		 = $config['ct_server_url'];
						$ct->server_ttl		 = $config['ct_server_ttl'];
						$ct->server_changed	 = $config['ct_server_changed'];
						$ct->stay_on_server	 = true;

						$ct_request				 = new CleantalkRequest();
						$ct_request->auth_key	 = $config['ct_auth_key'];
						$ct_request->feedback	 = $data['ct_request_id'] . ':' . $data['ct_sms_code'];

						$ct_result = $ct->sendFeedback($ct_request);

						$sms_error_text = (isset($ct_result->sms_error_text[$data['ct_request_id']])) ? $ct_result->sms_error_text[$data['ct_request_id']] : null;

						if ($ct_result->errno > 0 || $ct_result->sms[$data['ct_request_id']] == 0)
						{
							if ((isset($data['ct_attempt_send_code']) && $data['ct_attempt_send_code'] == 3) || $ct_result->errno > 0) {
								$data['ct_attempt_send_code'] = 0;
								$ct_request					 = new CleantalkRequest();
								$ct_request->auth_key		 = $config['ct_auth_key'];
								$ct_request->sender_email	 = $data['email'];
								$ct_request->sender_nickname = $data['username'];
								$ct_request->sender_ip		 = $data['sender_ip'];
								$ct_request->agent			 = 'ct-phpbb-' . preg_replace("/(\d)\.(\d)\.(\d)/", "$1$2$3", $config['ct_version']);
								$ct_request->response_lang	 = $config['ct_response_lang'];
								$ct_request->submit_time	 = $data['submit_time'];
								$ct_request->js_on			 = $data['js_on'];
								$ct_request->tz				 = $data['tz'];
								$ct_request->phone			 = $ct_phone;

								$ct					 = new Cleantalk();
								$ct->work_url		 = $config['ct_work_url'];
								$ct->server_url		 = $config['ct_server_url'];
								$ct->server_ttl		 = $config['ct_server_ttl'];
								$ct->server_changed	 = $config['ct_server_changed'];
								$ct->stay_on_server	 = false;

								$ct_result = $ct->isAllowUser($ct_request);

								if ($ct->server_change)
								{
									set_config('ct_work_url', $ct->work_url);
									set_config('ct_server_ttl', $ct->server_ttl);
									set_config('ct_server_changed', time());
								}

								$data['ct_request_id'] = $ct_result->id;
								$ct_data['data'] = $data;
								$ct_data['cp_data'] = $cp_data; 

								$sql = "UPDATE " . SESSIONS_TABLE . "
									SET ct_register_form_data = '" . $db->sql_escape(serialize($ct_data)) . "',
										ct_register_form_data_time = '" . time() . "'
									WHERE session_id = '" . $db->sql_escape($user->session_id) . "'";
								$db->sql_query($sql);

								if (isset($ct_result->sms_allow))
								{
									if ($ct_result->sms_allow == '-1')
									{
										$ct_activation_admin = true;
										$ct_activation_comment = $ct_result->sms_error_text;

										$config['email_enable'] = 1;
										$config['require_activation'] = USER_ACTIVATION_ADMIN;
									}
									elseif (isset($ct_result->sms_error_text) && !empty($ct_result->sms_error_text))
									{
										$error[] = $ct_result->sms_error_text;
										$template->assign_vars(array(
											'ERROR' => (sizeof($error)) ? implode('<br />', $error) : ''
										));
										$this->tpl_name	 = 'ucp_register';
									}
									else
									{
										if (!empty($sms_error_text))
										{
											$error[] = $sms_error_text;
										}
										else
										{
											$error[] = $user->lang['CT_SERVER_NOT_AVAILABLE'];
										}

										$template->assign_vars(array(
											'ERROR' => (sizeof($error)) ? implode('<br />', $error) : ''
										));

										$data['ct_request_id'] = $ct_result->id;

										$data = array_merge($data, array(
											'sender_ip' => $ct_request->sender_ip,
											'submit_time' => $ct_request->submit_time,
											'js_on' => $ct_request->js_on,
												));
										$ct_data['data'] = $data;
										$ct_data['cp_data'] = $cp_data; 

										$sql			 = "UPDATE " . SESSIONS_TABLE . "
										SET ct_register_form_data = '" . $db->sql_escape(serialize($ct_data)) . "',
											ct_register_form_data_time = '" . time() . "'
										WHERE session_id = '" . $db->sql_escape($user->session_id) . "'";
										$db->sql_query($sql);
										$this->tpl_name	 = 'ucp_check_sms';
										return;
									}
								}

								$this->tpl_name = 'ucp_register';
							} else {
								$data['ct_attempt_send_code'] = (isset($data['ct_attempt_send_code'])) ?
								(int)$data['ct_attempt_send_code']+1 : 1;
								$error[] = $sms_error_text;

								$template->assign_vars(array(
									'ERROR' => (sizeof($error)) ? implode('<br />', $error) : ''
								));
								$ct_data['data'] = $data;
								$ct_data['cp_data'] = $cp_data; 

								$sql			 = "UPDATE " . SESSIONS_TABLE . "
								SET ct_register_form_data = '" . $db->sql_escape(serialize($ct_data)) . "',
									ct_register_form_data_time = '" . time() . "'
								WHERE session_id = '" . $db->sql_escape($user->session_id) . "'";
								$db->sql_query($sql);
								$this->tpl_name	 = 'ucp_check_sms';
								return;
							}
						}

						if ($ct_result->sms[$data['ct_request_id']] == 1)
						{

							$sql			 = 'UPDATE ' . SESSIONS_TABLE . '
								SET ct_register_form_data = "",
									ct_register_form_data_time = "0"
								WHERE session_id = \'' . $db->sql_escape($user->session_id) . '\'';
							$db->sql_query($sql);
							$ct_result->id	 = $data['ct_request_id'];
						}

						if ($ct->server_change)
						{
							set_config('ct_work_url', $ct->work_url);
							set_config('ct_server_ttl', $ct->server_ttl);
							set_config('ct_server_changed', time());
						}
					}
				}
			}
			// End: cleantalk.ru moderation

			if (!sizeof($error))
			{
				$server_url = generate_board_url();

				// Which group by default?
				$group_name = ($coppa) ? 'REGISTERED_COPPA' : 'REGISTERED';

				$sql = 'SELECT group_id
					FROM ' . GROUPS_TABLE . "
					WHERE group_name = '" . $db->sql_escape($group_name) . "'
						AND group_type = " . GROUP_SPECIAL;
				$result = $db->sql_query($sql);
				$row = $db->sql_fetchrow($result);
				$db->sql_freeresult($result);

				if (!$row)
				{
					trigger_error('NO_GROUP');
				}

				$group_id = $row['group_id'];

				if (($coppa ||
					$config['require_activation'] == USER_ACTIVATION_SELF ||
					$config['require_activation'] == USER_ACTIVATION_ADMIN) && $config['email_enable'])
				{
					$user_actkey = gen_rand_string(mt_rand(6, 10));
					$user_type = USER_INACTIVE;
					$user_inactive_reason = INACTIVE_REGISTER;
					$user_inactive_time = time();
				}
				else
				{
					$user_type = USER_NORMAL;
					$user_actkey = '';
					$user_inactive_reason = 0;
					$user_inactive_time = 0;
				}

				$user_row = array(
					'username'				=> $data['username'],
					'user_password'			=> phpbb_hash($data['new_password']),
					'user_email'			=> $data['email'],
					'group_id'				=> (int) $group_id,
					'user_timezone'			=> (float) $data['tz'],
					'user_dst'				=> $is_dst,
					'user_lang'				=> $data['lang'],
					'user_type'				=> $user_type,
					'user_actkey'			=> $user_actkey,
					'user_ip'				=> $user->ip,
					'user_regdate'			=> time(),
					'user_inactive_reason'	=> $user_inactive_reason,
					'user_inactive_time'	=> $user_inactive_time,
				);
				// Begin: cleantalk.ru moderation
				if (isset($ct_result->id))
				{
					$ct_phone = ($ct_phone === 'not_filled' || $ct_phone === null) ? '' : $ct_phone;
					$user_row = array_merge(array(
						'ct_request_id' => $ct_result->id,
						'ct_phone_number' => $ct_phone,
							), $user_row);
				}
				// End: cleantalk.ru moderation

				if ($config['new_member_post_limit'])
				{
					$user_row['user_new'] = 1;
				}

				// Register user...
				$user_id = user_add($user_row, $cp_data);

				// This should not happen, because the required variables are listed above...
				if ($user_id === false)
				{
					trigger_error('NO_USER', E_USER_ERROR);
				}

				// Okay, captcha, your job is done.
				if ($config['enable_confirm'] && isset($captcha))
				{
					$captcha->reset();
				}

				if ($coppa && $config['email_enable'])
				{
					$message = $user->lang['ACCOUNT_COPPA'];
					$email_template = 'coppa_welcome_inactive';
				}
				else if ($config['require_activation'] == USER_ACTIVATION_SELF && $config['email_enable'])
				{
					$message = $user->lang['ACCOUNT_INACTIVE'];
					$email_template = 'user_welcome_inactive';
				}
				else if ($config['require_activation'] == USER_ACTIVATION_ADMIN && $config['email_enable'])
				{
					$message = $user->lang['ACCOUNT_INACTIVE_ADMIN'];
					$email_template = 'admin_welcome_inactive';
				}
				else
				{
					$message = $user->lang['ACCOUNT_ADDED'];
					$email_template = 'user_welcome';
				}

				if ($config['email_enable'])
				{
					include_once($phpbb_root_path . 'includes/functions_messenger.' . $phpEx);

					$messenger = new messenger(false);

					$messenger->template($email_template, $data['lang']);

					$messenger->to($data['email'], $data['username']);

					$messenger->anti_abuse_headers($config, $user);

					$messenger->assign_vars(array(
						'WELCOME_MSG'	=> htmlspecialchars_decode(sprintf($user->lang['WELCOME_SUBJECT'], $config['sitename'])),
						'USERNAME'		=> htmlspecialchars_decode($data['username']),
						'PASSWORD'		=> htmlspecialchars_decode($data['new_password']),
						'U_ACTIVATE'	=> "$server_url/ucp.$phpEx?mode=activate&u=$user_id&k=$user_actkey")
					);

					if ($coppa)
					{
						$messenger->assign_vars(array(
							'FAX_INFO'		=> $config['coppa_fax'],
							'MAIL_INFO'		=> $config['coppa_mail'],
							'EMAIL_ADDRESS'	=> $data['email'])
						);
					}

					$messenger->send(NOTIFY_EMAIL);

					if ($config['require_activation'] == USER_ACTIVATION_ADMIN)
					{
						// Grab an array of user_id's with a_user permissions ... these users can activate a user
						$admin_ary = $auth->acl_get_list(false, 'a_user', false);
						$admin_ary = (!empty($admin_ary[0]['a_user'])) ? $admin_ary[0]['a_user'] : array();

						// Also include founders
						$where_sql = ' WHERE user_type = ' . USER_FOUNDER;

						if (sizeof($admin_ary))
						{
							$where_sql .= ' OR ' . $db->sql_in_set('user_id', $admin_ary);
						}

						$sql = 'SELECT user_id, username, user_email, user_lang, user_jabber, user_notify_type
							FROM ' . USERS_TABLE . ' ' .
							$where_sql;
						$result = $db->sql_query($sql);

						while ($row = $db->sql_fetchrow($result))
						{
							$messenger->template('admin_activate', $row['user_lang']);
							$messenger->to($row['user_email'], $row['username']);
							$messenger->im($row['user_jabber'], $row['username']);

							$messenger->assign_vars(array(
								'USERNAME'			=> htmlspecialchars_decode($data['username']),
								'U_USER_DETAILS'	=> "$server_url/memberlist.$phpEx?mode=viewprofile&u=$user_id",
								'U_ACTIVATE'		=> "$server_url/ucp.$phpEx?mode=activate&u=$user_id&k=$user_actkey")
							);



							// Begin: cleantalk.ru moderation
							if (isset($ct_activation_admin))
							{
								$messenger->assign_vars(array(
									'EMAIL_SIG'			=> $ct_activation_comment
								));
							}
							// End: cleantalk.ru moderation
							$messenger->send($row['user_notify_type']);
						}
						$db->sql_freeresult($result);
					}
				}

				$message = $message . '<br /><br />' . sprintf($user->lang['RETURN_INDEX'], '<a href="' . append_sid("{$phpbb_root_path}index.$phpEx") . '">', '</a>');
				trigger_error($message);
			}
		}



		// Begin: cleantalk.ru moderation
		$sql = "UPDATE " . SESSIONS_TABLE . "
			SET ct_submit_time = '" . time() . "'
			WHERE session_id = '" . $db->sql_escape($user->session_id) . "'";
		$db->sql_query($sql);
		// End: cleantalk.ru moderation
		$s_hidden_fields = array(
			'agreed'		=> 'true',
			'change_lang'	=> 0,
		);

		if ($config['coppa_enable'])
		{
			$s_hidden_fields['coppa'] = $coppa;
		}

		if ($config['enable_confirm'])
		{
			$s_hidden_fields = array_merge($s_hidden_fields, $captcha->get_hidden_fields());
		}
		$s_hidden_fields = build_hidden_fields($s_hidden_fields);
		$confirm_image = '';

		// Visual Confirmation - Show images
		if ($config['enable_confirm'])
		{
			$template->assign_vars(array(
				'CAPTCHA_TEMPLATE'		=> $captcha->get_template(),
			));
		}

		//
		$l_reg_cond = '';
		switch ($config['require_activation'])
		{
			case USER_ACTIVATION_SELF:
				$l_reg_cond = $user->lang['UCP_EMAIL_ACTIVATE'];
			break;

			case USER_ACTIVATION_ADMIN:
				$l_reg_cond = $user->lang['UCP_ADMIN_ACTIVATE'];
			break;
		}

		$template->assign_vars(array(
			'ERROR'				=> (sizeof($error)) ? implode('<br />', $error) : '',
			'USERNAME'			=> $data['username'],
			'PASSWORD'			=> $data['new_password'],
			'PASSWORD_CONFIRM'	=> $data['password_confirm'],
			'EMAIL'				=> $data['email'],
			'EMAIL_CONFIRM'		=> $data['email_confirm'],

			'L_REG_COND'				=> $l_reg_cond,
			'L_USERNAME_EXPLAIN'		=> sprintf($user->lang[$config['allow_name_chars'] . '_EXPLAIN'], $config['min_name_chars'], $config['max_name_chars']),
			'L_PASSWORD_EXPLAIN'		=> sprintf($user->lang[$config['pass_complex'] . '_EXPLAIN'], $config['min_pass_chars'], $config['max_pass_chars']),

			'S_LANG_OPTIONS'	=> language_select($data['lang']),
			'S_TZ_OPTIONS'		=> tz_select($data['tz']),
			'S_CONFIRM_REFRESH'	=> ($config['enable_confirm'] && $config['confirm_refresh']) ? true : false,
			'S_REGISTRATION'	=> true,
			'S_COPPA'			=> $coppa,
			'S_HIDDEN_FIELDS'	=> $s_hidden_fields,
			'S_UCP_ACTION'		=> append_sid("{$phpbb_root_path}ucp.$phpEx", 'mode=register'),
		));

		//
		$user->profile_fields = array();

		// Generate profile fields -> Template Block Variable profile_fields
		$cp->generate_profile_fields('register', $user->get_iso_lang_id());

		//
		$this->tpl_name = 'ucp_register';
		$this->page_title = 'UCP_REGISTRATION';
	}
}

?>